@@ -2,9 +2,13 @@ package fileintegrity
 
 import (
 	"context"
+	"errors"
 	"fmt"
 	"reflect"
 	"strconv"
+	"strings"
+
+	"k8s.io/apimachinery/pkg/labels"
 
 	"k8s.io/apimachinery/pkg/api/resource"
 
@@ -27,23 +31,24 @@ import (
 
 	fileintegrityv1alpha1 "github.com/openshift/file-integrity-operator/pkg/apis/fileintegrity/v1alpha1"
 	"github.com/openshift/file-integrity-operator/pkg/common"
+	"github.com/openshift/file-integrity-operator/pkg/controller/metrics"
 )
 
 var log = logf.Log.WithName("controller_fileintegrity")
 
 // Add creates a new FileIntegrity Controller and adds it to the Manager. The Manager will set fields on the Controller
 // and Start it when the Manager is Started.
-func Add(mgr manager.Manager) error {
-	return add(mgr, newReconciler(mgr))
+func Add(mgr manager.Manager, met *metrics.Metrics) error {
+	return add(mgr, newReconciler(mgr, met), met)
 }
 
 // newReconciler returns a new reconcile.Reconciler
-func newReconciler(mgr manager.Manager) reconcile.Reconciler {
-	return &ReconcileFileIntegrity{client: mgr.GetClient(), scheme: mgr.GetScheme()}
+func newReconciler(mgr manager.Manager, met *metrics.Metrics) reconcile.Reconciler {
+	return &ReconcileFileIntegrity{client: mgr.GetClient(), scheme: mgr.GetScheme(), metrics: met}
 }
 
 // add adds a new Controller to mgr with r as the reconcile.Reconciler
-func add(mgr manager.Manager, r reconcile.Reconciler) error {
+func add(mgr manager.Manager, r reconcile.Reconciler, met *metrics.Metrics) error {
 	// Create a new controller
 	c, err := controller.New("fileintegrity-controller", mgr, controller.Options{Reconciler: r})
 	if err != nil {
@@ -74,8 +79,9 @@ var _ reconcile.Reconciler = &ReconcileFileIntegrity{}
 type ReconcileFileIntegrity struct {
 	// This client, initialized using mgr.Client() above, is a split client
 	// that reads objects from the cache and writes to the apiserver
-	client client.Client
-	scheme *runtime.Scheme
+	client  client.Client
+	scheme  *runtime.Scheme
+	metrics *metrics.Metrics
 }
 
 // handleDefaultConfigMaps creates the inital configMaps needed by the operator and aide pods. It returns the
@@ -148,30 +154,88 @@ func (r *ReconcileFileIntegrity) handleDefaultConfigMaps(logger logr.Logger, f *
 	return confCm, scriptsUpdated, nil
 }
 
-func (r *ReconcileFileIntegrity) createReinitDaemonSet(instance *fileintegrityv1alpha1.FileIntegrity) error {
+// narrowNodeSelector returns a LabelSelector as-is from instance if node == nil, otherwise, return a LabelSelector for
+// node if node is contained in the node group. Used for making a node-specific DS.
+func (r *ReconcileFileIntegrity) narrowNodeSelector(instance *fileintegrityv1alpha1.FileIntegrity, node *corev1.Node) (metav1.LabelSelector, error) {
+	narrowSelector := metav1.LabelSelector{}
+	if node == nil {
+		narrowSelector.MatchLabels = instance.Spec.NodeSelector
+		return narrowSelector, nil
+	}
+
+	hostnameFromLabel, ok := node.Labels["kubernetes.io/hostname"]
+	if !ok || len(hostnameFromLabel) == 0 {
+		return narrowSelector, errors.New("couldn't find hostname label from node")
+	}
+
+	narrowSelector.MatchLabels = map[string]string{
+		"kubernetes.io/hostname": hostnameFromLabel,
+	}
+
+	return narrowSelector, nil
+}
+
+// createReinitDaemonSet creates a re-init daemonSet. The daemonSet will be a single-node reinit if node is not "" (like
+// in the node update case), otherwise, it will apply to the whole node instance node group (like in the manual re-init
+// case).
+func (r *ReconcileFileIntegrity) createReinitDaemonSet(instance *fileintegrityv1alpha1.FileIntegrity, node string) error {
 	daemonSet := &appsv1.DaemonSet{}
-	dsName := common.GetReinitDaemonSetName(instance.Name)
+	dsName := common.ReinitDaemonSetNodeName(instance.Name, node)
 	dsNamespace := common.FileIntegrityNamespace
 
-	err := r.client.Get(context.TODO(), types.NamespacedName{Name: dsName, Namespace: dsNamespace}, daemonSet)
-	if err == nil {
+	getErr := r.client.Get(context.TODO(), types.NamespacedName{Name: dsName, Namespace: dsNamespace}, daemonSet)
+	if getErr == nil {
 		// Exists, so continue.
 		return nil
 	}
-
-	if !kerr.IsNotFound(err) {
-		return err
+	if !kerr.IsNotFound(getErr) {
+		return getErr
 	}
 
-	ds := reinitAideDaemonset(common.GetReinitDaemonSetName(instance.Name), instance)
+	reinitNode, tryNodeErr := r.tryGettingNode(node)
+	if tryNodeErr != nil && !kerr.IsNotFound(tryNodeErr) {
+		return tryNodeErr
+	}
+	if kerr.IsNotFound(tryNodeErr) {
+		// We wanted a node, but it was not found. It might be scaled down. Return with no error so we don't continue with
+		// creating a re-init daemonSet.
+		return nil
+	}
+
+	// The reinit daemonset may need to apply to a single node only.
+	selectorOverride, narrowErr := r.narrowNodeSelector(instance, reinitNode)
+	if narrowErr != nil {
+		return narrowErr
+	}
+
+	ds := reinitAideDaemonset(common.ReinitDaemonSetNodeName(instance.Name, node), instance, selectorOverride)
 	if err := controllerutil.SetControllerReference(instance, ds, r.scheme); err != nil {
 		return err
 	}
 
-	return r.client.Create(context.TODO(), ds)
+	createErr := r.client.Create(context.TODO(), ds)
+	if createErr == nil {
+		r.metrics.IncFileIntegrityReinitDaemonsetUpdate()
+	}
+
+	return createErr
 }
 
-func (r *ReconcileFileIntegrity) updateAideConfig(conf *corev1.ConfigMap, data string) error {
+// If node == "", return nil, nil, otherwise try to find a node with that name. Not found needs to be handled outside
+// of this function.
+func (r *ReconcileFileIntegrity) tryGettingNode(node string) (*corev1.Node, error) {
+	if len(node) == 0 {
+		return nil, nil
+	}
+	n := corev1.Node{}
+	err := r.client.Get(context.TODO(), types.NamespacedName{Name: node}, &n)
+	if err != nil {
+		return nil, err
+	}
+	return &n, nil
+}
+
+func (r *ReconcileFileIntegrity) updateAideConfig(conf *corev1.ConfigMap, data, node string) error {
 	confCopy := conf.DeepCopy()
 	confCopy.Data[common.DefaultConfDataKey] = data
 
@@ -180,12 +244,12 @@ func (r *ReconcileFileIntegrity) updateAideConfig(conf *corev1.ConfigMap, data s
 	}
 
 	// Mark the configMap as updated by the user-provided config, for the configMap-controller to trigger an update.
-	confCopy.Annotations[common.AideConfigUpdatedAnnotationKey] = "true"
+	confCopy.Annotations[common.AideConfigUpdatedAnnotationKey] = node
 
 	return r.client.Update(context.TODO(), confCopy)
 }
 
-func (r *ReconcileFileIntegrity) retrieveAndAnnotateAideConfig(conf *corev1.ConfigMap) error {
+func (r *ReconcileFileIntegrity) retrieveAndAnnotateAideConfig(conf *corev1.ConfigMap, node string) error {
 	cachedconf := &corev1.ConfigMap{}
 	// Get the latest config...
 	err := r.client.Get(context.TODO(), types.NamespacedName{Name: conf.Name, Namespace: conf.Namespace}, cachedconf)
@@ -193,7 +257,7 @@ func (r *ReconcileFileIntegrity) retrieveAndAnnotateAideConfig(conf *corev1.Conf
 		return err
 	}
 
-	return r.updateAideConfig(cachedconf, cachedconf.Data[common.DefaultConfDataKey])
+	return r.updateAideConfig(cachedconf, cachedconf.Data[common.DefaultConfDataKey], node)
 }
 
 func (r *ReconcileFileIntegrity) aideConfigIsDefault(instance *fileintegrityv1alpha1.FileIntegrity) (bool, error) {
@@ -222,7 +286,7 @@ func (r *ReconcileFileIntegrity) reconcileUserConfig(instance *fileintegrityv1al
 		if !hasDefaultConfig {
 			// The configuration was previously replaced. We want to restore it now.
 			reqLogger.Info("Restoring the AIDE configuration defaults.")
-			if err := r.updateAideConfig(currentConfig, DefaultAideConfig); err != nil {
+			if err := r.updateAideConfig(currentConfig, DefaultAideConfig, ""); err != nil {
 				return false, err
 			}
 			return true, nil
@@ -270,7 +334,7 @@ func (r *ReconcileFileIntegrity) reconcileUserConfig(instance *fileintegrityv1al
 		return false, nil
 	}
 
-	if err := r.updateAideConfig(currentConfig, preparedConf); err != nil {
+	if err := r.updateAideConfig(currentConfig, preparedConf, ""); err != nil {
 		return false, err
 	}
 
@@ -299,7 +363,7 @@ func (r *ReconcileFileIntegrity) Reconcile(request reconcile.Request) (reconcile
 		return reconcile.Result{}, err
 	}
 
-	daemonSetName := common.GetDaemonSetName(instance.Name)
+	daemonSetName := common.DaemonSetName(instance.Name)
 
 	defaultAideConf, scriptsUpdated, err := r.handleDefaultConfigMaps(reqLogger, instance)
 	if err != nil {
@@ -317,29 +381,33 @@ func (r *ReconcileFileIntegrity) Reconcile(request reconcile.Request) (reconcile
 		return reconcile.Result{}, err
 	}
 
-	_, forceReinit := instance.Annotations[common.AideDatabaseReinitAnnotationKey]
+	nodeToReinit := ""
+	nodeToReinit, forceReinit := instance.Annotations[common.AideDatabaseReinitAnnotationKey]
 	if hasNewConfig || forceReinit {
-		if forceReinit {
-			reqLogger.Info("Re-init forced, creating daemonSet.")
-		} else {
-			reqLogger.Info("Re-init triggered by configuration change, creating daemonSet.")
-		}
-		// This daemonset re-inits the database
-		// TODO(jaosorior): Add status about the re-init happening.
-		if err := r.createReinitDaemonSet(instance); err != nil {
+		if err := r.createReinitDaemonSet(instance, nodeToReinit); err != nil {
 			return reconcile.Result{}, err
 		}
+
+		if forceReinit {
+			reqLogger.Info("re-init daemonSet created, triggered by demand or node", "node", nodeToReinit)
+			r.metrics.IncFileIntegrityReinitByDemand()
+		} else {
+			reqLogger.Info("re-init daemonSet created, triggered by configuration change")
+			r.metrics.IncFileIntegrityReinitByConfig()
+		}
 	}
 
 	// Remove re-init annotation
 	if forceReinit {
 		reqLogger.Info("Annotating AIDE config to be updated.")
-		if err := r.retrieveAndAnnotateAideConfig(defaultAideConf); err != nil {
+		if err := r.retrieveAndAnnotateAideConfig(defaultAideConf, nodeToReinit); err != nil {
 			return reconcile.Result{}, err
 		}
 		fiCopy := instance.DeepCopy()
 		delete(fiCopy.Annotations, common.AideDatabaseReinitAnnotationKey)
 		reqLogger.Info("Removing re-init annotation.")
+		// TODO: We need to retry gracefully here (not leave the annotation active, causing a doubled metric in the
+		// forceReinit case)
 		if err := r.client.Update(context.TODO(), fiCopy); err != nil {
 			return reconcile.Result{}, err
 		}
@@ -352,6 +420,11 @@ func (r *ReconcileFileIntegrity) Reconcile(request reconcile.Request) (reconcile
 			reqLogger.Error(err, "error getting daemonSet")
 			return reconcile.Result{}, err
 		}
+
+		if legacyDeleteErr := r.deleteLegacyDaemonSets(instance); legacyDeleteErr != nil {
+			return reconcile.Result{}, legacyDeleteErr
+		}
+
 		// create
 		ds := aideDaemonset(daemonSetName, instance)
 
@@ -360,10 +433,14 @@ func (r *ReconcileFileIntegrity) Reconcile(request reconcile.Request) (reconcile
 			return reconcile.Result{}, err
 		}
 
-		if createErr := r.client.Create(context.TODO(), ds); createErr != nil && !kerr.IsAlreadyExists(createErr) {
+		createErr := r.client.Create(context.TODO(), ds)
+		if createErr != nil && !kerr.IsAlreadyExists(createErr) {
 			reqLogger.Error(createErr, "error creating daemonSet")
 			return reconcile.Result{}, createErr
 		}
+		if !kerr.IsAlreadyExists(createErr) {
+			r.metrics.IncFileIntegrityDaemonsetUpdate()
+		}
 	} else {
 		dsCopy := daemonSet.DeepCopy()
 		argsNeedUpdate := updateDSArgs(dsCopy, instance, reqLogger)
@@ -377,18 +454,41 @@ func (r *ReconcileFileIntegrity) Reconcile(request reconcile.Request) (reconcile
 				return reconcile.Result{}, err
 			}
 
+			r.metrics.IncFileIntegrityDaemonsetUpdate()
+
 			// TODO: We might want to change this to something that signals to the daemonSet pods that they need to
 			// gracefully exit, and let them restart that way.
-			err := common.RestartFileIntegrityDs(r.client, common.GetDaemonSetName(instance.Name))
+			err := common.RestartFileIntegrityDs(r.client, common.DaemonSetName(instance.Name))
 			if err != nil {
 				return reconcile.Result{}, err
 			}
 			reqLogger.Info("FileIntegrity daemon configuration changed - pods restarted.")
+			r.metrics.IncFileIntegrityDaemonsetPodKill()
 		}
 	}
 	return reconcile.Result{}, nil
 }
 
+// The old daemonSets had a "aide-ds-" prefix, but that is no longer. If any are around after upgrade, delete them.
+func (r *ReconcileFileIntegrity) deleteLegacyDaemonSets(instance *fileintegrityv1alpha1.FileIntegrity) error {
+	daemonSetList := &appsv1.DaemonSetList{}
+	if err := r.client.List(context.TODO(), daemonSetList, &client.ListOptions{LabelSelector: labels.SelectorFromSet(labels.Set{
+		common.IntegrityOwnerLabelKey: instance.Name,
+	})}); err != nil {
+		return err
+	}
+	for i, _ := range daemonSetList.Items {
+		daemonSet := &daemonSetList.Items[i]
+		// Check for the old prefixed ds, delete it (it's being replaced by the newly named ones.)
+		if strings.HasPrefix(daemonSet.Name, "aide-ds-") {
+			if deleteErr := r.client.Delete(context.TODO(), daemonSet); deleteErr != nil {
+				return deleteErr
+			}
+		}
+	}
+	return nil
+}
+
 func updateDSNodeSelector(currentDS *appsv1.DaemonSet, fi *fileintegrityv1alpha1.FileIntegrity, logger logr.Logger) bool {
 	nsRef := &currentDS.Spec.Template.Spec.NodeSelector
 	expectedNS := fi.Spec.NodeSelector
@@ -460,7 +560,8 @@ func defaultAIDEConfigMap(name string) *corev1.ConfigMap {
 			Name:      name,
 			Namespace: common.FileIntegrityNamespace,
 			Labels: map[string]string{
-				common.AideConfigLabelKey: "",
+				common.AideConfigLabelKey:     "",
+				common.IntegrityOwnerLabelKey: name,
 			},
 		},
 		Data: map[string]string{
@@ -509,7 +610,7 @@ func configMapKeyDataMatches(cm1, cm2 *corev1.ConfigMap, key string) bool {
 
 // reinitAideDaemonset returns a DaemonSet that runs a one-shot pod on each node. This pod touches a file
 // on the host OS that informs the AIDE daemon to back up and reinitialize the AIDE db.
-func reinitAideDaemonset(reinitDaemonSetName string, fi *fileintegrityv1alpha1.FileIntegrity) *appsv1.DaemonSet {
+func reinitAideDaemonset(reinitDaemonSetName string, fi *fileintegrityv1alpha1.FileIntegrity, selector metav1.LabelSelector) *appsv1.DaemonSet {
 	priv := true
 	runAs := int64(0)
 	mode := int32(0744)
@@ -518,6 +619,9 @@ func reinitAideDaemonset(reinitDaemonSetName string, fi *fileintegrityv1alpha1.F
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      reinitDaemonSetName,
 			Namespace: common.FileIntegrityNamespace,
+			Labels: map[string]string{
+				common.IntegrityReinitOwnerLabelKey: fi.Name,
+			},
 		},
 		Spec: appsv1.DaemonSetSpec{
 			Selector: &metav1.LabelSelector{
@@ -532,7 +636,7 @@ func reinitAideDaemonset(reinitDaemonSetName string, fi *fileintegrityv1alpha1.F
 					},
 				},
 				Spec: corev1.PodSpec{
-					NodeSelector:       fi.Spec.NodeSelector,
+					NodeSelector:       selector.MatchLabels,
 					Tolerations:        fi.Spec.Tolerations,
 					ServiceAccountName: common.OperatorServiceAccountName,
 					InitContainers: []corev1.Container{
